/*
 * Copyright (C) 2011 Canonical Ltd
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 3 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Authored by Mikkel Kamstrup Erlandsen <mikkel.kamstrup@canonical.com>
 *
 */

using GLib;
using Config;

namespace Unity.ApplicationsPlace {

  static Application? app = null;
  static Daemon? daemon = null;

  public static int main (string[] args)
  {
    /* Sort up locale to get translations but also sorting and
     * punctuation right */
    GLib.Intl.textdomain (Config.PACKAGE);
    GLib.Intl.bindtextdomain (Config.PACKAGE, Config.LOCALEDIR);
    GLib.Intl.bind_textdomain_codeset (Config.PACKAGE, "UTF-8");
    GLib.Intl.setlocale(GLib.LocaleCategory.ALL, "");
  
    /* Make sure the desktop appinfos are picked up correctly */
    DesktopAppInfo.set_desktop_env ("GNOME");
  
    /* Use GApplication directly for single instance app functionality */
    app = new Application ("com.canonical.Unity.ApplicationsPlace",
                           ApplicationFlags.IS_SERVICE);
    try {
      app.register ();
    } catch (Error e) {
      /* FIXME: We get this error if another daemon is already running,
       * but it uses a generic error so we can't detect this reliably... */
      print ("Failed to start applications daemon: %s\n", e.message);
      return 1;
    }
    
    if (app.get_is_remote ())
      {
        print ("Another instance of the Unity Applications Daemon" +
               "already appears to be running.\nBailing out.\n");
        return 2;
      }
    
    /* Hold()ing the app makes sure the GApplication doesn't exit */
    daemon = new Daemon ();
    app.hold();
    return app.run ();
  }
  
} /* namespace */
